# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Report the information about current device and platform
#' @details Return a list containing the information about current device and platform.
#' The list object includes device name, palatform information, toolkit information,
#' and compute capability.
#' @return List containing the information about device and platform.
#' @export
#' @references \url{http://arrayfire.org/docs/group__device__func__prop.htm#gaa9eab56ddac21650581682912ea78293}
arrayfire_device_info <- function() {
    .Call(`_RcppArrayFire_arrayfire_device_info`)
}

#' Switch to specified device
#' @param index the index of a device switched to.
#' @details Available indices
#' can be obtained by calling \code{\link{arrayfire_count_device}}.
#' @export
#' @references \url{http://arrayfire.org/docs/group__device__func__set.htm#gafbb906ca5b89ec43fdb0e3a14d1df1e7}
arrayfire_set_device <- function(index) {
    invisible(.Call(`_RcppArrayFire_arrayfire_set_device`, index))
}

#' Return the index of current device
#' @details Get the index of current device which is lower or
#' equal to a value returned by \code{\link{arrayfire_count_device}}.
#' @return the index of current device.
#' @export
#' @references \url{http://arrayfire.org/docs/group__device__func__get.htm#ga4dfe3f90475b735384f8b28cf2b19a11}
arrayfire_get_device <- function() {
    .Call(`_RcppArrayFire_arrayfire_get_device`)
}

#' Return the number of available devices
#' @details Return the number of available devices.
#' @return the number of available devices.
#' @export
#' @references \url{http://arrayfire.org/docs/group__device__func__count.htm#gac3c8750da69ee883fd14c0a8290e45f1}
arrayfire_count_device <- function() {
    .Call(`_RcppArrayFire_arrayfire_count_device`)
}

#' Display ArrayFire and device info
#' @param verbose  Flag to return verbose info
#' @details Display ArrayFire and device info.
#' @export
#' @references \url{http://arrayfire.org/docs/group__device__func__info.htm#ga1360e706b046b1c5b299bd6e689a0276} and \url{http://arrayfire.org/docs/group__device__func__info__string.htm#ga73ee21947f018445ea720036c1edfb69}
arrayfire_info <- function(verbose = FALSE) {
    invisible(.Call(`_RcppArrayFire_arrayfire_info`, verbose))
}

#' Set the seed for random number generation in ArrayFire.
#' @details Sets the seed for the current default random engine in ArrayFire. Note that a (64bit) double is used as input, since R does not support 64bit integers.
#' @param seed a (64bit) double whose bit representation is reinterpreted as a 64bit integer
#' @export
#' @references \url{http://arrayfire.org/docs/group__random__func__set__seed.htm}
arrayfire_set_seed <- function(seed) {
    invisible(.Call(`_RcppArrayFire_arrayfire_set_seed`, seed))
}

#' Get the seed for random number generation in ArrayFire.
#' @details Gets the seed for the current default random engine in ArrayFire. Note that a (64bit) double is used as output, since R does not support 64bit integers.
#' @return a (64bit) double whose bit representation is reinterpreted from the 64bit integer seed
#' @export
#' @references \url{http://arrayfire.org/docs/group__random__func__get__seed.htm}
arrayfire_get_seed <- function() {
    .Call(`_RcppArrayFire_arrayfire_get_seed`)
}

#' Get string description for the active backend
#' @details Get string description for the active backend.
#' @return string description for the active backend
#' @export
#' @references \url{http://arrayfire.org/docs/group__unified__func__getactivebackend.htm#gac6a8e976a151d007e0cf5cf4f11da2a9}
arrayfire_get_active_backend <- function() {
    .Call(`_RcppArrayFire_arrayfire_get_active_backend`)
}

#' Get a vector of string descriptions for the available backends
#' @details Get a vector of string descriptions for the available backends.
#' @return vector of string descriptions for the available backends
#' @export
#' @references \url{http://arrayfire.org/docs/group__unified__func__getavailbackends.htm#ga32fc807a8d4680987a915362573c1a2c}
arrayfire_get_available_backends <- function() {
    .Call(`_RcppArrayFire_arrayfire_get_available_backends`)
}

#' Set the current backend
#' @details Set the current backend.
#' @param backend string description of backend (DEFAULT, CPU, OPENCL, CUDA)
#' @export
#' @references \url{http://arrayfire.org/docs/group__unified__func__setbackend.htm#ga7e0fad1f134b0421811ee3a4ce47e987}
arrayfire_set_backend <- function(backend = "DEFAULT") {
    invisible(.Call(`_RcppArrayFire_arrayfire_set_backend`, backend))
}

#' Fast and simple linear model
#' @details Single precision floats are used by default since not all devices support double precision.
#' @param X  a model matrix
#' @param y  a vector containing the explained variable
#' @param useDouble  use float or double internally
#' @return \code{fastLmPure} returns a list with three components:
#' \describe{
#'   \item{\code{coefficients}}{a vector of coefficients}
#'   \item{\code{residuals}}{the residuals, that is response minus fitted values}
#'   \item{\code{fitted.values}}{the fitted mean values}
#'   \item{\code{stderr}}{a vector of the (estimated) standard errors of the coefficient estimates}
#'   \item{\code{df.residual}}{a scalar denoting the degrees of freedom in the model}
#'   \item{\code{intercept}}{a boolean denoting if a model with intercept was fitted}
#' }
#' @export
fastLmPure <- function(X, y, useDouble = FALSE) {
    .Call(`_RcppArrayFire_fastLmPure`, X, y, useDouble)
}

#' Fast runif alternative
#' @details Fast generation of uniformly distributed random values. The default is to use single precision,
#'  since not all devices support calculations using double precision.
#' @param n  number of observations
#' @param min  lower limit of the distribution
#' @param max  upper limit of the distribution
#' @param useDouble  use float or double internally
#' @return A numeric vector of random values
#' @seealso \code{\link{runif}} and \url{http://arrayfire.org/docs/group__random__func__randu.htm}
#' @export
fastRunif <- function(n, min = 0, max = 1, useDouble = FALSE) {
    .Call(`_RcppArrayFire_fastRunif`, n, min, max, useDouble)
}

#' Fast rnorm alternative
#' @details Fast generation of normaly distributed random values. The default is to use single precision,
#'  since not all devices support calculations using double precision.
#' @param n  number of observations
#' @param mean  mean value of the distribution
#' @param sd  standard deviation of the distribution
#' @param useDouble  use float or double internally
#' @return A numeric vector of random values
#' @seealso \code{\link{rnorm}} and \url{http://arrayfire.org/docs/group__random__func__randn.htm}
#' @export
fastRnorm <- function(n, mean = 0, sd = 1, useDouble = FALSE) {
    .Call(`_RcppArrayFire_fastRnorm`, n, mean, sd, useDouble)
}

